@@ -9,10 +9,12 @@
 #include <stddef.h>
 #include "tss2_tpm2_types.h"
 #include "tss2-sys/sysapi_util.h"
-#ifdef OSSL
+#if defined(OSSL)
 #include "esys_crypto_ossl.h"
+#elif defined(MBED)
+#include "esys_crypto_mbed.h"
 #else
-#include "esys_crypto_gcrypt.h"
+#error "No crypto backend defined"
 #endif
 
 #ifdef __cplusplus
