@@ -35,23 +35,6 @@ cmp_UINT16(const UINT16 * in1, const UINT16 * in2)
     }
 }
 
-/**
- * Compare variables of type BYTE.
- * @param[in] in1 Variable to be compared with:
- * @param[in] in2
- */
-static bool
-cmp_BYTE(const BYTE * in1, const BYTE * in2)
-{
-    LOG_TRACE("call");
-    if (*in1 == *in2)
-        return true;
-    else {
-        LOG_TRACE("cmp false");
-        return false;
-    }
-}
-
 /**
  * Compare two arrays of type BYTE.
  * @param[in] in1 array to be compared with:.
@@ -65,12 +48,12 @@ cmp_BYTE_array(const BYTE * in1, size_t count1, const BYTE * in2, size_t count2)
         LOG_TRACE("cmp false");
         return false;
     }
-    for (size_t i = 0; i < count1; i++) {
-        if (!cmp_BYTE(&in1[i], &in2[i])) {
-            LOG_TRACE("cmp false");
-            return false;
-        }
+
+    if (memcmp(in1, in2, count2) != 0) {
+        LOG_TRACE("cmp false");
+        return false;
     }
+
     return true;
 }
 
@@ -168,6 +151,7 @@ iesys_DeleteAllResourceObjects(ESYS_CONTEXT * esys_context)
         next_node_rsrc = node_rsrc->next;
         SAFE_FREE(node_rsrc);
     }
+    esys_context->rsrc_list = NULL;
 }
 /**  Compute the TPM nonce of the session used for parameter encryption.
  *
@@ -402,9 +386,40 @@ iesys_handle_to_tpm_handle(ESYS_TR esys_handle, TPM2_HANDLE * tpm_handle)
         *tpm_handle = TPM2_RH_PLATFORM_NV;
         return TPM2_RC_SUCCESS;
     }
+    if (esys_handle >= ESYS_TR_RH_ACT_FIRST &&
+        esys_handle <= ESYS_TR_RH_ACT_LAST) {
+        *tpm_handle = TPM2_RH_ACT_0 + (esys_handle - ESYS_TR_RH_ACT_FIRST);
+        return TPM2_RC_SUCCESS;
+    }
     LOG_ERROR("Error: Esys invalid ESAPI handle (%x).", esys_handle);
     return TSS2_ESYS_RC_BAD_VALUE;
 }
+
+/**
+ * Determines if an ESYS_TR (UINT32) is assigned a raw TPM2_HANDLE (UINT32)
+ * hierarchy type.
+ *
+ * @param handle [in] The handle to check if it's a hierarchy or not.
+ * @return
+ *  true if it is a hierarchy, false otherwise.
+ */
+bool
+iesys_is_platform_handle(ESYS_TR handle) {
+
+    switch(handle) {
+    case TPM2_RH_OWNER:
+    case TPM2_RH_PLATFORM:
+    case TPM2_RH_PLATFORM_NV:
+    case TPM2_RH_ENDORSEMENT:
+    case TPM2_RH_NULL:
+        LOG_WARNING("Convert handle from TPM2_RH to ESYS_TR, got: 0x%x",
+                handle);
+        return true;
+    default:
+        return false;
+    }
+}
+
 /** Get the type of a tpm handle.
  *
  * @parm handle[in] The tpm handle.
@@ -563,10 +578,8 @@ iesys_gen_caller_nonces(ESYS_CONTEXT * esys_context)
  *
  * In case where command does not support param encryption/decryption
  * store the original session attributes and update them accordingly.
- * Return true is command support param encryption.
  *
- * @retval TRUE if command support param encryption
- * @retval FLASE if command does not support param encryption
+ * @retval void
  */
 static void
 iesys_update_session_flags(ESYS_CONTEXT * esys_context,
@@ -722,7 +735,6 @@ iesys_encrypt_param(ESYS_CONTEXT * esys_context,
                                                  symDef->algorithm,
                                                  symDef->keyBits.aes,
                                                  symDef->mode.aes,
-                                                 AES_BLOCK_SIZE_IN_BYTES,
                                                  &encrypt_buffer[0], paramSize,
                                                  &symKey[aes_off]);
                 return_if_error(r, "AES encryption not possible");
@@ -822,7 +834,6 @@ iesys_decrypt_param(ESYS_CONTEXT * esys_context)
                                      symDef->algorithm,
                                      symDef->keyBits.aes,
                                      symDef->mode.aes,
-                                     AES_BLOCK_SIZE_IN_BYTES,
                                      &plaintext[0], p2BSize,
                                      &symKey[aes_off]);
         return_if_error(r, "Decryption error");
@@ -1245,7 +1256,7 @@ iesys_compute_hmac(RSRC_NODE_T * session,
  * the command authorization are computed.
  * @param[in] esys_context The esys context to issue the command on.
  * @param[in] h1-3 The esys session resource objects.
- * @param[out] The list if the authorizations with teh computed HMACs.
+ * @param[out] The list if the authorizations with the computed HMACs.
  * @param[out] auth The computed HMAC value.
  * @retval TSS2_RC_SUCCESS on success.
  * @retval TSS2_ESYS_RC_MEMORY Memory can not be allocated.
@@ -1282,6 +1293,18 @@ iesys_gen_auths(ESYS_CONTEXT * esys_context,
                                     &encryptNonce);
     return_if_error(r, "More than one crypt session");
 
+    /*
+     * TPM2.0 Architecture 19.6.5 Note 7
+     *
+     * If the same session (not the first session) is used for decrypt and
+     * encrypt, its nonceTPM is only used once. If different sessions are used
+     * for decrypt and encrypt, both nonceTPMs are included
+     */
+    if (decryptNonceIdx && (decryptNonceIdx == encryptNonceIdx)) {
+        decryptNonceIdx = 0;
+    }
+
+
     /* Compute cp hash values for command buffer for all used algorithms */
 
     r = iesys_compute_cp_hashtab(esys_context,
@@ -1299,7 +1322,7 @@ iesys_gen_auths(ESYS_CONTEXT * esys_context,
                 auths->auths[auths->count].hmac.size = 0;
                 auths->count += 1;
             } else {
-                auths->auths[auths->count].sessionHandle = TPM2_RS_PW;
+                auths->auths[auths->count].sessionHandle = TPM2_RH_PW;
                 auths->auths[auths->count].hmac = objects[session_idx]->auth;
                 auths->count += 1;
             }
@@ -1446,23 +1469,28 @@ iesys_nv_get_name(TPM2B_NV_PUBLIC * publicInfo, TPM2B_NAME * name)
     r = Tss2_MU_TPMS_NV_PUBLIC_Marshal(&publicInfo->nvPublic,
                                        &buffer[0], sizeof(TPMS_NV_PUBLIC),
                                        &offset);
-    return_if_error(r, "Marshaling TPMS_NV_PUBLIC");
+    goto_if_error(r, "Marshaling TPMS_NV_PUBLIC", error_cleanup);
 
     r = iesys_crypto_hash_update(cryptoContext, &buffer[0], offset);
-    return_if_error(r, "crypto hash update");
+    goto_if_error(r, "crypto hash update", error_cleanup);
 
     r = iesys_crypto_hash_finish(&cryptoContext, &name->name[len_alg_id],
                                      &size);
-    return_if_error(r, "crypto hash finish");
+    goto_if_error(r, "crypto hash finish", error_cleanup);
 
     offset = 0;
     r = Tss2_MU_TPMI_ALG_HASH_Marshal(publicInfo->nvPublic.nameAlg,
                                   &name->name[0], sizeof(TPMI_ALG_HASH),
                                   &offset);
-    return_if_error(r, "Marshaling TPMI_ALG_HASH");
+    goto_if_error(r, "Marshaling TPMI_ALG_HASH", error_cleanup);
 
     name->size = size + len_alg_id;
     return TSS2_RC_SUCCESS;
+
+error_cleanup:
+    if (cryptoContext)
+        iesys_crypto_hash_abort(&cryptoContext);
+    return r;
 }
 
 /** Compute the name of a TPM transient or persistent object.
@@ -1494,23 +1522,28 @@ iesys_get_name(TPM2B_PUBLIC * publicInfo, TPM2B_NAME * name)
 
     r = Tss2_MU_TPMT_PUBLIC_Marshal(&publicInfo->publicArea,
                                     &buffer[0], sizeof(TPMT_PUBLIC), &offset);
-    return_if_error(r, "Marshaling TPMT_PUBLIC");
+    goto_if_error(r, "Marshaling TPMT_PUBLIC", error_cleanup);
 
     r = iesys_crypto_hash_update(cryptoContext, &buffer[0], offset);
-    return_if_error(r, "crypto hash update");
+    goto_if_error(r, "crypto hash update", error_cleanup);
 
     r = iesys_crypto_hash_finish(&cryptoContext, &name->name[len_alg_id],
                                      &size);
-    return_if_error(r, "crypto hash finish");
+    goto_if_error(r, "crypto hash finish", error_cleanup);
 
     offset = 0;
     r = Tss2_MU_TPMI_ALG_HASH_Marshal(publicInfo->publicArea.nameAlg,
                                   &name->name[0], sizeof(TPMI_ALG_HASH),
                                   &offset);
-    return_if_error(r, "Marshaling TPMI_ALG_HASH");
+    goto_if_error(r, "Marshaling TPMI_ALG_HASH", error_cleanup);
 
     name->size = size + len_alg_id;
     return TSS2_RC_SUCCESS;
+
+error_cleanup:
+    if (cryptoContext)
+        iesys_crypto_hash_abort(&cryptoContext);
+    return r;
 }
 
 /** Check whether the return code corresponds to an TPM error.
@@ -1529,3 +1562,55 @@ iesys_tpm_error(TSS2_RC r)
              (r & TSS2_RC_LAYER_MASK) == TSS2_RESMGR_TPM_RC_LAYER ||
              (r & TSS2_RC_LAYER_MASK) == TSS2_RESMGR_RC_LAYER));
 }
+
+
+/** Replace auth value with Hash for long auth values.
+ *
+ * if the size of auth value exceeds hash_size the auth value
+ * will be replaced with the hash of the auth value.
+ *
+ * @param[in,out] auth_value The auth value to be adapted.
+ * @param[in] hash_alg The hash alg used for adaption.
+ * @retval TSS2_RC_SUCCESS if the function call was a success.
+ * @retval TSS2_ESYS_RC_BAD_VALUE if an invalid hash is passed.
+ * @retval TSS2_ESYS_RC_MEMORY if the ESAPI cannot allocate enough memory.
+ * @retval TSS2_ESYS_RC_GENERAL_FAILURE for a failure during digest
+ *         computation.
+ */
+TSS2_RC
+iesys_hash_long_auth_values(
+    TPM2B_AUTH *auth_value,
+    TPMI_ALG_HASH hash_alg)
+{
+    TSS2_RC r;
+    IESYS_CRYPTO_CONTEXT_BLOB *cryptoContext;
+    TPM2B_AUTH hash2b;
+    size_t hash_size;
+
+    r = iesys_crypto_hash_get_digest_size(hash_alg, &hash_size);
+    return_if_error(r, "Get digest size.");
+
+    if (auth_value && auth_value->size > hash_size) {
+        /* The auth value has to be adapted. */
+        r = iesys_crypto_hash_start(&cryptoContext, hash_alg);
+        return_if_error(r, "crypto hash start");
+
+        r = iesys_crypto_hash_update(cryptoContext, &auth_value->buffer[0],
+                                     auth_value->size);
+        goto_if_error(r, "crypto hash update", error_cleanup);
+
+        r = iesys_crypto_hash_finish(&cryptoContext, &hash2b.buffer[0],
+                                     &hash_size);
+        goto_if_error(r, "crypto hash finish", error_cleanup);
+
+        memcpy(&auth_value->buffer[0], &hash2b.buffer[0], hash_size);
+        auth_value->size = hash_size;
+    }
+    return r;
+
+ error_cleanup:
+    if (cryptoContext) {
+        iesys_crypto_hash_abort(&cryptoContext);
+    }
+    return r;
+}
