@@ -1805,7 +1805,7 @@ ifapi_get_quote_info(
     TSS2_RC r;
     size_t offset = 0;
 
-    jso = json_tokener_parse(quoteInfo);
+    jso = ifapi_parse_json(quoteInfo);
     return_if_null(jso, "Json error.", TSS2_FAPI_RC_BAD_VALUE);
 
     memset(&fapi_quote_info->attest.attested.quote.pcrSelect, 0,
@@ -2106,6 +2106,9 @@ ifapi_calculate_pcr_digest(
     case TPM2_ALG_ECDSA:
         pcr_digest_hash_alg = quote_info->sig_scheme.details.ecdsa.hashAlg;
         break;
+    case TPM2_ALG_SM2:
+        pcr_digest_hash_alg = quote_info->sig_scheme.details.sm2.hashAlg;
+        break;
     default:
         LOG_ERROR("Unknown sig scheme");
         return TSS2_FAPI_RC_BAD_VALUE;
@@ -2501,6 +2504,9 @@ ifapi_get_curl_buffer(unsigned char * url, unsigned char ** buffer,
                           size_t *buffer_size) {
     int ret = -1;
     struct CurlBufferStruct curl_buffer = { .size = 0, .buffer = NULL };
+#ifdef CURLU_ALLOW_SPACE
+    CURLU *urlp = NULL;
+#endif
 
     CURLcode rc = curl_global_init(CURL_GLOBAL_DEFAULT);
     if (rc != CURLE_OK) {
@@ -2514,7 +2520,24 @@ ifapi_get_curl_buffer(unsigned char * url, unsigned char ** buffer,
         goto out_global_cleanup;
     }
 
+#ifdef CURLU_ALLOW_SPACE
+    urlp = curl_url();
+    if (!urlp) {
+        LOG_ERROR("curl_url failed.");
+        goto out_easy_cleanup;
+    }
+    CURLUcode url_rc;
+    url_rc = curl_url_set(urlp, CURLUPART_URL, (const char *)url, CURLU_ALLOW_SPACE | CURLU_URLENCODE);
+    if (url_rc) {
+        LOG_ERROR("curl_url_set for CURUPART_URL failed: %s",
+                  curl_url_strerror(url_rc));
+        goto out_easy_cleanup;
+    }
+    rc = curl_easy_setopt(curl, CURLOPT_CURLU, urlp);
+#else
     rc = curl_easy_setopt(curl, CURLOPT_URL, url);
+#endif
+
     if (rc != CURLE_OK) {
         LOG_ERROR("curl_easy_setopt for CURLOPT_URL failed: %s",
                 curl_easy_strerror(rc));
@@ -2537,6 +2560,13 @@ ifapi_get_curl_buffer(unsigned char * url, unsigned char ** buffer,
         goto out_easy_cleanup;
     }
 
+    rc = curl_easy_setopt(curl, CURLOPT_FOLLOWLOCATION, 1);
+    if (rc != CURLE_OK) {
+        LOG_ERROR("curl_easy_setopt for CURLOPT_FOLLOWLOCATION failed: %s",
+                  curl_easy_strerror(rc));
+        goto out_easy_cleanup;
+    }
+
     if (LOGMODULE_status == LOGLEVEL_TRACE) {
         if (CURLE_OK != curl_easy_setopt(curl, CURLOPT_VERBOSE, 1L)) {
             LOG_WARNING("Curl easy setopt verbose failed");
@@ -2555,6 +2585,10 @@ ifapi_get_curl_buffer(unsigned char * url, unsigned char ** buffer,
     ret = 0;
 
 out_easy_cleanup:
+#ifdef CURLU_ALLOW_SPACE
+    if (urlp)
+        curl_url_cleanup(urlp);
+#endif
     if (ret != 0)
         free(curl_buffer.buffer);
     curl_easy_cleanup(curl);
