@@ -70,6 +70,9 @@ ifapi_json_TPMA_LOCALITY_serialize(const TPMA_LOCALITY in, json_object **jso);
 TSS2_RC
 ifapi_json_TPMA_CC_serialize(const TPMA_CC in, json_object **jso);
 
+TSS2_RC
+ifapi_json_TPMA_ACT_serialize(const TPMA_ACT in, json_object **jso);
+
 TSS2_RC
 ifapi_json_TPMI_YES_NO_serialize(const TPMI_YES_NO in, json_object **jso);
 
@@ -159,6 +162,13 @@ TSS2_RC
 ifapi_json_TPMS_TAGGED_PCR_SELECT_serialize(const TPMS_TAGGED_PCR_SELECT *in,
         json_object **jso);
 
+TSS2_RC
+ifapi_json_TPMS_TAGGED_POLICY_serialize(const TPMS_TAGGED_POLICY *in,
+                                        json_object **jso);
+
+TSS2_RC
+ifapi_json_TPMS_ACT_DATA_serialize(const TPMS_ACT_DATA *in, json_object **jso);
+
 TSS2_RC
 ifapi_json_TPML_CC_serialize(const TPML_CC *in, json_object **jso);
 
@@ -192,6 +202,13 @@ TSS2_RC
 ifapi_json_TPML_ECC_CURVE_serialize(const TPML_ECC_CURVE *in,
                                     json_object **jso);
 
+TSS2_RC
+ifapi_json_TPML_TAGGED_POLICY_serialize(const TPML_TAGGED_POLICY *in,
+                                        json_object **jso);
+
+TSS2_RC
+ifapi_json_TPML_ACT_DATA_serialize(const TPML_ACT_DATA *in, json_object **jso);
+
 TSS2_RC
 ifapi_json_TPMU_CAPABILITIES_serialize(const TPMU_CAPABILITIES *in,
                                        UINT32 selector, json_object **jso);
