@@ -208,8 +208,10 @@ TSS2_RC Tss2_MU_##type##_Marshal(type const *src, uint8_t buffer[], \
         return rc; \
 \
     /* Update the size to the real value */ \
-    if (buffer) \
-        *(UINT16 *)ptr = HOST_TO_BE_16(buffer + local_offset - ptr - 2); \
+    if (buffer) { \
+        UINT16 t = HOST_TO_BE_16(buffer + local_offset - ptr - 2); \
+        memcpy(ptr, &t, sizeof(t)); \
+    } \
 \
     if (offset != NULL) { \
         *offset = local_offset; \
