@@ -13,14 +13,14 @@
 #include "tss2_esys.h"
 
 #include "esys_iutil.h"
-#include "test-esapi.h"
+#include "test-esys.h"
 #define LOGMODULE test
 #include "util/log.h"
 #include "util/aux_util.h"
 
-/** Test the ESAPI function Esys_FieldUpgradeStart and   Esys_FieldUpgradeData.
+/** Test the ESYS function Esys_FieldUpgradeStart and   Esys_FieldUpgradeData.
  *
- * Tested ESAPI commands:
+ * Tested ESYS commands:
  *  - Esys_FieldUpgradeData() (O)
  *  - Esys_FieldUpgradeStart() (O)
  *
@@ -86,6 +86,6 @@ test_esys_field_upgrade(ESYS_CONTEXT * esys_context)
 }
 
 int
-test_invoke_esapi(ESYS_CONTEXT * esys_context) {
+test_invoke_esys(ESYS_CONTEXT * esys_context) {
     return test_esys_field_upgrade(esys_context);
 }
