@@ -13,14 +13,14 @@
 #include "tss2_esys.h"
 
 #include "esys_iutil.h"
-#include "test-esapi.h"
+#include "test-esys.h"
 #define LOGMODULE test
 #include "util/log.h"
 #include "util/aux_util.h"
 
-/** Test the ESAPI function Esys_FirmwareRead.
+/** Test the ESYS function Esys_FirmwareRead.
  *
- * Tested ESAPI commands:
+ * Tested ESYS commands:
  *  - Esys_FirmwareRead() (O)
  *
  * @param[in,out] esys_context The ESYS_CONTEXT.
@@ -60,6 +60,6 @@ test_esys_firmware_read(ESYS_CONTEXT * esys_context)
 }
 
 int
-test_invoke_esapi(ESYS_CONTEXT * esys_context) {
+test_invoke_esys(ESYS_CONTEXT * esys_context) {
     return test_esys_firmware_read(esys_context);
 }
