@@ -17,10 +17,10 @@
 #include "util/log.h"
 #include "util/aux_util.h"
 
-/** Test the ESAPI commands HashSequenceStart, SequenceUpdate,
+/** Test the ESYS commands HashSequenceStart, SequenceUpdate,
  *  and SequenceComplete.
  *
- * Tested ESAPI commands:
+ * Tested ESYS commands:
  *  - Esys_FlushContext() (M)
  *  - Esys_HashSequenceStart() (M)
  *  - Esys_SequenceComplete() (M)
@@ -30,15 +30,19 @@
  * Used compiler defines: TEST_SESSION
  *
  * @param[in,out] esys_context The ESYS_CONTEXT.
+ * @param[in] hierarchy the hierarchy to start the hash sequence in.
  * @retval EXIT_FAILURE
  * @retval EXIT_SUCCESS
  */
 
 int
-test_esys_hashsequencestart(ESYS_CONTEXT * esys_context)
+test_esys_hashsequencestart(ESYS_CONTEXT * esys_context, ESYS_TR hierarchy)
 {
     TSS2_RC r;
 
+    TPM2B_DIGEST *result = NULL;
+    TPMT_TK_HASHCHECK *validation = NULL;
+
 #ifdef TEST_SESSION
     ESYS_TR session = ESYS_TR_NONE;
     TPMT_SYM_DEF symmetric = {.algorithm = TPM2_ALG_AES,
@@ -57,7 +61,7 @@ test_esys_hashsequencestart(ESYS_CONTEXT * esys_context)
     r = Esys_StartAuthSession(esys_context, ESYS_TR_NONE, ESYS_TR_NONE,
                               ESYS_TR_NONE, ESYS_TR_NONE, ESYS_TR_NONE,
                               &nonceCaller,
-                              TPM2_SE_HMAC, &symmetric, TPM2_ALG_SHA1,
+                              TPM2_SE_HMAC, &symmetric, TPM2_ALG_SHA256,
                               &session);
     goto_if_error(r, "Error: During initialization of session", error);
 #endif /* TEST_SESSION */
@@ -99,9 +103,6 @@ test_esys_hashsequencestart(ESYS_CONTEXT * esys_context)
                             );
     goto_if_error(r, "Error: SequenceUpdate", error);
 
-    TPM2B_DIGEST *result;
-    TPMT_TK_HASHCHECK *validation;
-
     r = Esys_SequenceComplete(esys_context,
                               sequenceHandle_handle,
 #ifdef TEST_SESSION
@@ -112,7 +113,7 @@ test_esys_hashsequencestart(ESYS_CONTEXT * esys_context)
                               ESYS_TR_NONE,
                               ESYS_TR_NONE,
                               &buffer,
-                              TPM2_RH_OWNER,
+                              hierarchy,
                               &result,
                               &validation
                               );
@@ -123,6 +124,8 @@ test_esys_hashsequencestart(ESYS_CONTEXT * esys_context)
     goto_if_error(r, "Error: FlushContext", error);
 #endif
 
+    Esys_Free(result);
+    Esys_Free(validation);
     return EXIT_SUCCESS;
 
  error:
@@ -134,10 +137,20 @@ test_esys_hashsequencestart(ESYS_CONTEXT * esys_context)
         }
     }
 #endif
+    Esys_Free(result);
+    Esys_Free(validation);
     return EXIT_FAILURE;
 }
 
 int
-test_invoke_esapi(ESYS_CONTEXT * esys_context) {
-    return test_esys_hashsequencestart(esys_context);
+test_invoke_esys(ESYS_CONTEXT * esys_context) {
+    int rc = test_esys_hashsequencestart(esys_context, ESYS_TR_RH_OWNER);
+    if (rc)
+        return rc;
+
+    /*
+     * Test that backwards compat API change is still working, see:
+     *   - https://github.com/tpm2-software/tpm2-tss/issues/1750
+     */
+    return test_esys_hashsequencestart(esys_context, TPM2_RH_OWNER);
 }
