@@ -17,11 +17,11 @@
 #include "util/log.h"
 #include "util/aux_util.h"
 
-/** Test the ESAPI commands: HMAC_Start, SequenceUpdate, and SequenceComplete.
+/** Test the ESYS commands: HMAC_Start, SequenceUpdate, and SequenceComplete.
  *
  * The HMAC key is created by using Esys_CreatePrimary.
  *
- * Tested ESAPI commands:
+ * Tested ESYS commands:
  *  - Esys_CreatePrimary() (M)
  *  - Esys_FlushContext() (M)
  *  - Esys_HMAC_Start() (M)
@@ -42,6 +42,14 @@ test_esys_hmacsequencestart(ESYS_CONTEXT * esys_context)
     TSS2_RC r;
     ESYS_TR primaryHandle = ESYS_TR_NONE;
 
+    TPM2B_PUBLIC *outPublic = NULL;
+    TPM2B_CREATION_DATA *creationData = NULL;
+    TPM2B_DIGEST *creationHash = NULL;
+    TPMT_TK_CREATION *creationTicket = NULL;
+
+    TPM2B_DIGEST *result = NULL;
+    TPMT_TK_HASHCHECK *validation = NULL;
+
 #ifdef TEST_SESSION
     ESYS_TR session = ESYS_TR_NONE;
     TPMT_SYM_DEF symmetric = {.algorithm = TPM2_ALG_AES,
@@ -60,7 +68,7 @@ test_esys_hmacsequencestart(ESYS_CONTEXT * esys_context)
     r = Esys_StartAuthSession(esys_context, ESYS_TR_NONE, ESYS_TR_NONE,
                               ESYS_TR_NONE, ESYS_TR_NONE, ESYS_TR_NONE,
                               &nonceCaller,
-                              TPM2_SE_HMAC, &symmetric, TPM2_ALG_SHA1,
+                              TPM2_SE_HMAC, &symmetric, TPM2_ALG_SHA256,
                               &session);
 
     goto_if_error(r, "Error: During initialization of session", error);
@@ -95,18 +103,13 @@ test_esys_hmacsequencestart(ESYS_CONTEXT * esys_context)
         .count = 0,
     };
 
-    TPM2B_PUBLIC *outPublic;
-    TPM2B_CREATION_DATA *creationData;
-    TPM2B_DIGEST *creationHash;
-    TPMT_TK_CREATION *creationTicket;
-
-    inPublic.publicArea.nameAlg = TPM2_ALG_SHA1;
+    inPublic.publicArea.nameAlg = TPM2_ALG_SHA256;
     inPublic.publicArea.type = TPM2_ALG_KEYEDHASH;
     inPublic.publicArea.objectAttributes |= TPMA_OBJECT_SIGN_ENCRYPT;
     inPublic.publicArea.objectAttributes |= TPMA_OBJECT_USERWITHAUTH;
     inPublic.publicArea.objectAttributes |= TPMA_OBJECT_SENSITIVEDATAORIGIN;
     inPublic.publicArea.parameters.keyedHashDetail.scheme.scheme = TPM2_ALG_HMAC;
-    inPublic.publicArea.parameters.keyedHashDetail.scheme.details.hmac.hashAlg = TPM2_ALG_SHA1;
+    inPublic.publicArea.parameters.keyedHashDetail.scheme.details.hmac.hashAlg = TPM2_ALG_SHA256;
 
     TPM2B_AUTH auth = {.size = 20,
                        .buffer={10, 11, 12, 13, 14, 15, 16, 17, 18, 19,
@@ -122,7 +125,7 @@ test_esys_hmacsequencestart(ESYS_CONTEXT * esys_context)
     r = Esys_TR_SetAuth(esys_context, primaryHandle, &authValuePrimary);
     goto_if_error(r, "Error: TR_SetAuth", error);
 
-    TPMI_ALG_HASH hashAlg = TPM2_ALG_SHA1;
+    TPMI_ALG_HASH hashAlg = TPM2_ALG_SHA256;
     ESYS_TR sequenceHandle;
 
     r = Esys_HMAC_Start(esys_context,
@@ -159,9 +162,6 @@ test_esys_hmacsequencestart(ESYS_CONTEXT * esys_context)
                             );
     goto_if_error(r, "Error: SequenceUpdate", error);
 
-    TPM2B_DIGEST *result;
-    TPMT_TK_HASHCHECK *validation;
-
     r = Esys_SequenceComplete(esys_context,
                               sequenceHandle,
 #ifdef TEST_SESSION
@@ -172,7 +172,7 @@ test_esys_hmacsequencestart(ESYS_CONTEXT * esys_context)
                               ESYS_TR_NONE,
                               ESYS_TR_NONE,
                               &buffer,
-                              TPM2_RH_OWNER,
+                              ESYS_TR_RH_OWNER,
                               &result,
                               &validation
                               );
@@ -183,13 +183,16 @@ test_esys_hmacsequencestart(ESYS_CONTEXT * esys_context)
     goto_if_error(r, "Error: FlushContext", error);
 #endif
 
+    Esys_Free(result);
+    Esys_Free(validation);
+
     /* Check HMAC_Start with auth equal NULL */
 
 #ifdef TEST_SESSION
     r = Esys_StartAuthSession(esys_context, ESYS_TR_NONE, ESYS_TR_NONE,
                               ESYS_TR_NONE, ESYS_TR_NONE, ESYS_TR_NONE,
                               &nonceCaller,
-                              TPM2_SE_HMAC, &symmetric, TPM2_ALG_SHA1,
+                              TPM2_SE_HMAC, &symmetric, TPM2_ALG_SHA256,
                               &session);
 
     goto_if_error(r, "Error: During initialization of session", error);
@@ -233,7 +236,7 @@ test_esys_hmacsequencestart(ESYS_CONTEXT * esys_context)
                               ESYS_TR_NONE,
                               ESYS_TR_NONE,
                               &buffer,
-                              TPM2_RH_OWNER,
+                              ESYS_TR_RH_OWNER,
                               &result,
                               &validation
                               );
@@ -247,6 +250,12 @@ test_esys_hmacsequencestart(ESYS_CONTEXT * esys_context)
     goto_if_error(r, "Error: FlushContext", error);
 #endif
 
+    Esys_Free(outPublic);
+    Esys_Free(creationData);
+    Esys_Free(creationHash);
+    Esys_Free(creationTicket);
+    Esys_Free(result);
+    Esys_Free(validation);
     return EXIT_SUCCESS;
 
  error:
@@ -265,10 +274,16 @@ test_esys_hmacsequencestart(ESYS_CONTEXT * esys_context)
     }
 #endif
 
+    Esys_Free(outPublic);
+    Esys_Free(creationData);
+    Esys_Free(creationHash);
+    Esys_Free(creationTicket);
+    Esys_Free(result);
+    Esys_Free(validation);
     return EXIT_FAILURE;
 }
 
 int
-test_invoke_esapi(ESYS_CONTEXT * esys_context) {
+test_invoke_esys(ESYS_CONTEXT * esys_context) {
     return test_esys_hmacsequencestart(esys_context);
 }
