@@ -22,7 +22,7 @@
  *  Then we call Esys_TR_GetName to see if the correct public name has been
  * retrieved.
  *
- * Tested ESAPI commands:
+ * Tested ESYS commands:
  *  - Esys_NV_DefineSpace() (M)
  *  - Esys_NV_ReadPublic() (M)
  *  - Esys_NV_UndefineSpace() (M)
@@ -47,7 +47,7 @@ test_esys_tr_fromTpmPublic_nv(ESYS_CONTEXT * ectx)
         .size = 0,
         .nvPublic = {
             .nvIndex =TPM2_NV_INDEX_FIRST,
-            .nameAlg = TPM2_ALG_SHA1,
+            .nameAlg = TPM2_ALG_SHA256,
             .attributes = TPMA_NV_AUTHWRITE | TPMA_NV_AUTHREAD,
             .authPolicy = {
                  .size = 0,
@@ -116,6 +116,6 @@ error:
 }
 
 int
-test_invoke_esapi(ESYS_CONTEXT * esys_context) {
+test_invoke_esys(ESYS_CONTEXT * esys_context) {
     return test_esys_tr_fromTpmPublic_nv(esys_context);
 }
