@@ -147,6 +147,41 @@ test_fapi_quote(FAPI_CONTEXT *context)
 
     const char *log_check_list[] =
         {
+         "["
+         "  {"
+         "    \"recnum\":1,"
+         "    \"pcr\":16,"
+         "    \"digests\":["
+         "      {"
+         "        \"hashAlg\":\"SHA1\","
+         "        \"digest\":\"494179714a6cd627239dfededf2de9ef994caf03\""
+         "      },"
+         "      {"
+         "        \"hashAlg\":\"SHA256\","
+         "        \"digest\":\"1f825aa2f0020ef7cf91dfa30da4668d791c5d4824fc8e41354b89ec05795ab3\""
+         "      },"
+         "      {"
+         "        \"hashAlg\":\"SHA384\","
+         "        \"digest\":\"182e95266adff49059e706c61483478fe0688150c8d08b95fab5cfde961f12d903aaf44104af4ce72ba6a4bf20302b2e\""
+         "      },"
+         "      {"
+         "        \"hashAlg\":\"SHA512\","
+         "        \"digest\":\"0f89ee1fcb7b0a4f7809d1267a029719004c5a5e5ec323a7c3523a20974f9a3f202f56fadba4cd9e8d654ab9f2e96dc5c795ea176fa20ede8d854c342f903533\""
+         "      },"
+         "      {"
+         "        \"hashAlg\":\"SM3_256\","
+         "        \"digest\":\"24c898bdb4d258f9bebb2e820d4ed478a7c013b37bd9e5006515730c18a70416\""
+         "      }"
+         "    ],"
+         "    \"type\":\"tss2\","
+         "    \"sub_event\":{"
+         "      \"data\":\"00010203040506070809\","
+         "      \"event\":{"
+         "        \"test\":\"myfile\""
+         "      }"
+         "    }"
+         "  }"
+         "]",
          "["
          "  {"
          "    \"recnum\":1,"
@@ -270,6 +305,17 @@ test_fapi_quote(FAPI_CONTEXT *context)
     ASSERT(cmp_strtokens(pathlist, check_pathlist, ":"));
     LOG_INFO("\nPathlist: %s\n", check_pathlist);
 
+    /* Invalidate qualifying data */
+    qualifyingData[0] = 0;
+
+    r = Fapi_VerifyQuote(context, "HS/SRK/mySignKey",
+                         qualifyingData, 20,  quoteInfo,
+                         signature, signatureSize, log);
+    if (r == TPM2_RC_SUCCESS) {
+        LOG_ERROR("Invalid qualifying data was not detected.");
+        goto error;
+    }
+
     r = Fapi_Delete(context, "/");
     goto_if_error(r, "Error Fapi_Delete", error);
 
