@@ -13,10 +13,13 @@
 #include <string.h>
 #include <assert.h>
 
-#include <openssl/sha.h>
-#include <openssl/hmac.h>
 #include <openssl/evp.h>
-#include <openssl/opensslv.h>
+#include <openssl/sha.h>
+#if OPENSSL_VERSION_NUMBER < 0x30000000L
+#include <openssl/hmac.h>
+#else
+#include <openssl/core_names.h>
+#endif
 
 #define LOGMODULE testintegration
 #include "util/log.h"
@@ -45,7 +48,7 @@ create_primary_rsa_2048_aes_128_cfb (
     /* session parameters */
     /* command session info */
     TSS2L_SYS_AUTH_COMMAND  sessions_cmd = {
-        .auths = {{ .sessionHandle = TPM2_RS_PW }},
+        .auths = {{ .sessionHandle = TPM2_RH_PW }},
         .count = 1
     };
     /* response session info */
@@ -133,7 +136,7 @@ create_aes_128_cfb (
     /* session parameters */
     /* command session info */
     TSS2L_SYS_AUTH_COMMAND  sessions_cmd = {
-        .auths = {{ .sessionHandle = TPM2_RS_PW }},
+        .auths = {{ .sessionHandle = TPM2_RH_PW }},
         .count = 1
     };
     /* response session info */
@@ -204,7 +207,7 @@ create_keyedhash_key (
     /* session parameters */
     /* command session info */
     TSS2L_SYS_AUTH_COMMAND  sessions_cmd = {
-        .auths = {{ .sessionHandle = TPM2_RS_PW }},
+        .auths = {{ .sessionHandle = TPM2_RH_PW }},
         .count = 1
     };
     /* response session info */
@@ -249,7 +252,7 @@ tpm_encrypt_decrypt_cfb (
     TPM2B_MAX_BUFFER *data_in,
     TPM2B_MAX_BUFFER *data_out)
 {
-    TPMI_ALG_SYM_MODE mode = TPM2_ALG_NULL;
+    TPMI_ALG_CIPHER_MODE mode = TPM2_ALG_NULL;
     TPM2B_IV iv_in = TPM2B_IV_INIT;
     TPM2B_IV iv_out = TPM2B_IV_INIT;
 
@@ -257,7 +260,7 @@ tpm_encrypt_decrypt_cfb (
     /* command session info */
     /* command session info */
     TSS2L_SYS_AUTH_COMMAND  sessions_cmd = {
-        .auths = {{ .sessionHandle = TPM2_RS_PW }},
+        .auths = {{ .sessionHandle = TPM2_RH_PW }},
         .count = 1
     };
     /* response session info */
@@ -306,7 +309,7 @@ tpm_encrypt_decrypt_2_cfb (
     TPM2B_MAX_BUFFER *data_in,
     TPM2B_MAX_BUFFER *data_out)
 {
-    TPMI_ALG_SYM_MODE mode = TPM2_ALG_NULL;
+    TPMI_ALG_CIPHER_MODE mode = TPM2_ALG_NULL;
     TPM2B_IV iv_in = TPM2B_IV_INIT;
     TPM2B_IV iv_out = TPM2B_IV_INIT;
 
@@ -314,7 +317,7 @@ tpm_encrypt_decrypt_2_cfb (
     /* command session info */
     /* command session info */
     TSS2L_SYS_AUTH_COMMAND  sessions_cmd = {
-        .auths = {{ .sessionHandle = TPM2_RS_PW }},
+        .auths = {{ .sessionHandle = TPM2_RH_PW }},
         .count = 1
     };
     /* response session info */
@@ -451,6 +454,29 @@ encrypt_cfb (
     return encrypt_decrypt_cfb(data_out, data_in, NO, key, iv);
 }
 
+#if HAVE_EVP_SM3 && !defined(OPENSSL_NO_SM3)
+static unsigned char *SM3(const unsigned char *d, size_t n, unsigned char *md)
+{
+    EVP_MD_CTX *ctx;
+    static unsigned char m[TPM2_SM3_256_DIGEST_SIZE] = { 0 };
+    uint32_t mdLen = TPM2_SM3_256_DIGEST_SIZE;
+
+    if (md == NULL) {
+        md = m;
+    }
+    ctx = EVP_MD_CTX_new();
+    EVP_DigestInit_ex(ctx, EVP_sm3(), NULL);
+    EVP_DigestUpdate(ctx, d, n);
+    EVP_DigestFinal_ex(ctx, md, &mdLen);
+    if (mdLen != TPM2_SM3_256_DIGEST_SIZE) {
+        EVP_MD_CTX_free(ctx);
+        return NULL;
+    }
+    EVP_MD_CTX_free(ctx);
+    return md;
+}
+#endif
+
 TSS2_RC
 hash (
     TPM2_ALG_ID alg,
@@ -475,6 +501,12 @@ hash (
         SHA512(data, size, out->buffer);
         out->size = TPM2_SHA512_DIGEST_SIZE;
         break;
+#if HAVE_EVP_SM3 && !defined(OPENSSL_NO_SM3)
+    case TPM2_ALG_SM3_256:
+        SM3(data, size, out->buffer);
+        out->size = TPM2_SM3_256_DIGEST_SIZE;
+        break;
+#endif
     default:
         return TSS2_SYS_RC_BAD_VALUE;
     }
@@ -489,22 +521,18 @@ hmac(
     TPM2B_DIGEST **buffer_list,
     TPM2B_DIGEST *out)
 {
-#if OPENSSL_VERSION_NUMBER >= 0x10100000L
-    HMAC_CTX *ctx;
-#else
-    HMAC_CTX _ctx;
-    HMAC_CTX *ctx = &_ctx;
-#endif
-    EVP_MD *evp;
     int rc = 1, i;
-    unsigned int *buf = NULL, size;
+    unsigned int *buf = NULL;
     uint8_t *buf_ptr;
+    EVP_MD *evp;
 
-#if OPENSSL_VERSION_NUMBER >= 0x10100000L
-    /* HMAC_CTX_new and HMAC_CTX_free are new in openSSL 1.1.0 */
-    ctx = HMAC_CTX_new();
+#if OPENSSL_VERSION_NUMBER < 0x30000000L
+    unsigned int size;
+    HMAC_CTX *ctx = HMAC_CTX_new();
 #else
-    HMAC_CTX_init(ctx);
+    size_t size;
+    EVP_MAC *hmac = EVP_MAC_fetch(NULL, "HMAC", NULL);
+    EVP_MAC_CTX *ctx = EVP_MAC_CTX_new(hmac);
 #endif
 
     if (!ctx)
@@ -527,6 +555,12 @@ hmac(
         evp = (EVP_MD *) EVP_sha512();
         out->size = TPM2_SHA512_DIGEST_SIZE;
         break;
+#if HAVE_EVP_SM3 && !defined(OPENSSL_NO_SM3)
+    case TPM2_ALG_SM3_256:
+        evp = (EVP_MD *) EVP_sm3();
+        out->size = TPM2_SM3_256_DIGEST_SIZE;
+        break;
+#endif
     default:
         rc = TSS2_SYS_RC_BAD_VALUE;
         goto out;
@@ -538,21 +572,33 @@ hmac(
 
     buf_ptr = (uint8_t *)buf;
 
-#if OPENSSL_VERSION_NUMBER >= 0x10100000L
+#if OPENSSL_VERSION_NUMBER < 0x30000000L
     rc = HMAC_Init_ex(ctx, key, key_len, evp, NULL);
 #else
-    rc = HMAC_Init(ctx, key, key_len, evp);
-#endif
+    OSSL_PARAM params[2];
 
+    params[0] = OSSL_PARAM_construct_utf8_string(OSSL_ALG_PARAM_DIGEST,
+                                                 (char *)EVP_MD_get0_name(evp), 0);
+    params[1] = OSSL_PARAM_construct_end();
+    rc = EVP_MAC_init(ctx, key, key_len, params);
+#endif
     if (rc != 1)
         goto out;
     for (i = 0; buffer_list[i] != 0; i++) {
+#if OPENSSL_VERSION_NUMBER < 0x30000000L
         rc = HMAC_Update(ctx, buffer_list[i]->buffer, buffer_list[i]->size);
+#else
+        rc = EVP_MAC_update(ctx, buffer_list[i]->buffer, buffer_list[i]->size);
+#endif
         if (rc != 1)
             goto out;
     }
     /* buf_ptr has to be 4 bytes alligned for whatever reason */
+#if OPENSSL_VERSION_NUMBER < 0x30000000L
     rc = HMAC_Final(ctx, buf_ptr, &size);
+#else
+    rc = EVP_MAC_final(ctx, buf_ptr, &size, out->size);
+#endif
     if (rc != 1)
         goto out;
 
@@ -561,10 +607,11 @@ hmac(
     memcpy(out->buffer, buf, out->size);
 
 out:
-#if OPENSSL_VERSION_NUMBER >= 0x10100000L
+#if OPENSSL_VERSION_NUMBER < 0x30000000L
     HMAC_CTX_free(ctx);
 #else
-    HMAC_CTX_cleanup(ctx);
+    EVP_MAC_CTX_free(ctx);
+    EVP_MAC_free(hmac);
 #endif
 
     if (buf)
@@ -681,7 +728,7 @@ DefineNvIndex (
         .count = 1,
         .auths = {
             {
-                .sessionHandle = TPM2_RS_PW,
+                .sessionHandle = TPM2_RH_PW,
                 .sessionAttributes = 0,
                 .nonce = { .size = 0 },
                 .hmac = { .size = 0 },
