@@ -171,7 +171,7 @@ UINT16_unmarshal_buffer_null (void **state)
 void
 UINT16_unmarshal_dest_null (void **state)
 {
-    uint8_t buffer [1];
+    uint8_t buffer [1] = { 0 };
     TSS2_RC rc;
 
     rc = Tss2_MU_UINT16_Unmarshal (buffer, sizeof (buffer), NULL, NULL);
@@ -186,7 +186,7 @@ void
 UINT16_unmarshal_buffer_size_lt_offset (void **state)
 {
     UINT16   dest = 0;
-    uint8_t buffer [1];
+    uint8_t buffer [1] = { 0 };
     size_t  offset = sizeof (buffer) + 1;
     TSS2_RC rc;
 
@@ -204,7 +204,7 @@ void
 UINT16_unmarshal_buffer_size_lt_dest (void **state)
 {
     UINT16   dest = 0;
-    uint8_t buffer [1];
+    uint8_t buffer [1] = { 0 };
     size_t  offset = sizeof (buffer);
     TSS2_RC rc;
 
