@@ -22,6 +22,7 @@
 #define LOGMODULE tests
 #include "util/log.h"
 #include "util/aux_util.h"
+#include "esys-dummy-defs.h"
 
 /**
  * This unit test looks into a set of Esys_<cmd>() functions and tests the
@@ -35,132 +36,6 @@
 #define TCTI_TPMERROR_MAGIC 0x5441455252000000ULL        /* 'TAERR\0' */
 #define TCTI_TPMERROR_VERSION 0x1
 
-/*
- * Esys handles for dummy session and key objects, and initialization values for
- * other objects, which can be used in ESAPI test calls
- */
-#define DUMMY_TR_HANDLE_POLICY_SESSION  ESYS_TR_MIN_OBJECT
-#define DUMMY_TR_HANDLE_KEY ESYS_TR_MIN_OBJECT+1
-#define DUMMY_TR_HANDLE_NV_INDEX ESYS_TR_MIN_OBJECT+2
-#define DUMMY_TR_HANDLE_HIERARCHY_OWNER ESYS_TR_MIN_OBJECT+3
-#define DUMMY_TR_HANDLE_HIERARCHY_PLATFORM ESYS_TR_MIN_OBJECT+4
-#define DUMMY_TR_HANDLE_PRIVACY_ADMIN ESYS_TR_MIN_OBJECT+5
-#define DUMMY_TR_HANDLE_HMAC_SESSION  ESYS_TR_MIN_OBJECT+6
-#define DUMMY_TR_HANDLE_LOCKOUT ESYS_TR_MIN_OBJECT+7
-#define DUMMY_IN_PUBLIC_DATA { \
-        .size = 0, \
-        .publicArea = { \
-            .type = TPM2_ALG_ECC, \
-            .nameAlg = TPM2_ALG_SHA256, \
-            .objectAttributes = (TPMA_OBJECT_USERWITHAUTH | \
-                                 TPMA_OBJECT_RESTRICTED | \
-                                 TPMA_OBJECT_SIGN_ENCRYPT | \
-                                 TPMA_OBJECT_FIXEDTPM | \
-                                 TPMA_OBJECT_FIXEDPARENT | \
-                                 TPMA_OBJECT_SENSITIVEDATAORIGIN), \
-            .authPolicy = { \
-                 .size = 0, \
-             }, \
-            .parameters.eccDetail = { \
-                 .symmetric = { \
-                     .algorithm = \
-                     TPM2_ALG_NULL, \
-                     .keyBits.aes = \
-                     128, \
-                     .mode.aes = \
-                     TPM2_ALG_ECB, \
-                 }, \
-                 .scheme = { \
-                      .scheme = \
-                      TPM2_ALG_ECDSA, \
-                      .details = { \
-                          .ecdsa = \
-                          {. \
-                           hashAlg \
-                           = \
-                           TPM2_ALG_SHA256}}, \
-                  }, \
-                 .curveID = TPM2_ECC_NIST_P256, \
-                 .kdf = { \
-                      .scheme = TPM2_ALG_NULL, \
-                      .details = {}} \
-             }, \
-            .unique.ecc = { \
-                 .x = {.size = 0,.buffer = {}}, \
-                 .y = {.size = 0,.buffer = {}}, \
-             }, \
-        }, \
-    }
-
-#define DUMMY_TPMT_PUBLIC_PARAMS { \
-        .type = TPM2_ALG_ECC, \
-            .parameters.eccDetail = { \
-            .symmetric = { \
-                 .algorithm = \
-                 TPM2_ALG_NULL, \
-                 .keyBits.aes = \
-                 128, \
-                 .mode.aes = \
-                 TPM2_ALG_ECB, \
-             }, \
-            .scheme = { \
-                 .scheme = \
-                 TPM2_ALG_ECDSA, \
-                 .details = { \
-                     .ecdsa = \
-                     {. \
-                      hashAlg \
-                      = \
-                      TPM2_ALG_SHA256}}, \
-             }, \
-            .curveID = TPM2_ECC_NIST_P256, \
-            .kdf = { \
-                 .scheme = TPM2_ALG_NULL, \
-                 .details = {}} \
-        } \
-    }
-
-#define DUMMY_2B_DATA(NAME)  { \
-        .size = 20, \
-        NAME = {1, 2, 3, 4, 5, 6, 7, 8, 9, 10, \
-                   11, 12, 13, 14, 15, 16, 17, 18, 19, 20} \
-    }
-
-#define DUMMY_2B_DATA16(NAME)  { \
-        .size = 16, \
-        NAME = {1, 2, 3, 4, 5, 6, 7, 8, 9, 10, \
-                   11, 12, 13, 14, 15, 16 } \
-    }
-
-#define DUMMY_2B_DATA0 { \
-        .size = 0, \
-        .buffer = {}, \
-    }
-
-#define DUMMY_SYMMETRIC {.algorithm = TPM2_ALG_AES, \
-        .keyBits = {.aes = 128}, \
-        .mode = {.aes = TPM2_ALG_CFB} \
-    }
-
-#define DUMMY_TPMT_TK_AUTH { .tag = TPM2_ST_AUTH_SIGNED , .hierarchy = TPM2_RH_OWNER, .digest = {0} }
-
-#define DUMMY_TPMT_TK_CREATION { .tag = TPM2_ST_CREATION , .hierarchy = TPM2_RH_OWNER, .digest = {0} }
-
-#define DUMMY_TPMT_TK_VERIFIED { .tag = TPM2_ST_VERIFIED , .hierarchy = TPM2_RH_OWNER, .digest = {0} }
-
-#define DUMMY_TPMT_TK_HASHCHECK { .tag = TPM2_ST_HASHCHECK , .hierarchy = TPM2_RH_OWNER, .digest = {0} }
-
-#define DUMMY_RSA_DECRYPT { .scheme = TPM2_ALG_NULL }
-
-#define DUMMY_TPMT_SIGNATURE { \
-        .sigAlg = TPM2_ALG_RSAPSS, \
-        .signature = { \
-            .rsapss = { \
-                 .hash = TPM2_ALG_SHA1, .sig= {0} \
-             } \
-        } \
-    };
-
 typedef struct {
     uint64_t magic;
     uint32_t version;
@@ -178,9 +53,9 @@ static TSS2_RC
 tcti_tpmerror_transmit(TSS2_TCTI_CONTEXT * tctiContext,
                       size_t size, const uint8_t * buffer)
 {
-    (void)(tctiContext);
-    (void)(size);
-    (void)(buffer);
+    UNUSED(tctiContext);
+    UNUSED(size);
+    UNUSED(buffer);
 
     return TSS2_RC_SUCCESS;
 }
@@ -196,8 +71,8 @@ tcti_tpmerror_receive(TSS2_TCTI_CONTEXT * tctiContext,
                      size_t * response_size,
                      uint8_t * response_buffer, int32_t timeout)
 {
-    (void)(tctiContext);
-    (void) timeout;
+    UNUSED(tctiContext);
+    UNUSED(timeout);
 
     *response_size = sizeof(response);
     if (response_buffer != NULL)
@@ -209,7 +84,7 @@ tcti_tpmerror_receive(TSS2_TCTI_CONTEXT * tctiContext,
 static void
 tcti_tpmerror_finalize(TSS2_TCTI_CONTEXT * tctiContext)
 {
-    (void)(tctiContext);
+    UNUSED(tctiContext);
 }
 
 static TSS2_RC
@@ -520,7 +395,7 @@ test_LoadExternal(void **state)
                           ESYS_TR_NONE,
                           ESYS_TR_NONE,
                           ESYS_TR_NONE,
-                          NULL, &inPublic, TPM2_RH_OWNER, &objectHandle_handle);
+                          NULL, &inPublic, ESYS_TR_RH_OWNER, &objectHandle_handle);
 
     assert_int_equal(r, 0x0FFF);
 }
@@ -849,7 +724,7 @@ test_EncryptDecrypt(void **state)
 
     ESYS_TR keyHandle_handle = DUMMY_TR_HANDLE_KEY;
     TPMI_YES_NO decrypt = 0;
-    TPMI_ALG_SYM_MODE mode = TPM2_ALG_NULL;
+    TPMI_ALG_CIPHER_MODE mode = TPM2_ALG_NULL;
     TPM2B_IV ivIn = DUMMY_2B_DATA16(.buffer);
     TPM2B_MAX_BUFFER inData = DUMMY_2B_DATA(.buffer);
     TPM2B_MAX_BUFFER *outData;
@@ -875,7 +750,7 @@ test_EncryptDecrypt2(void **state)
     ESYS_TR keyHandle_handle = DUMMY_TR_HANDLE_KEY;
     TPM2B_MAX_BUFFER inData = DUMMY_2B_DATA(.buffer);
     TPMI_YES_NO decrypt = 0;
-    TPMI_ALG_SYM_MODE mode = TPM2_ALG_NULL;
+    TPMI_ALG_CIPHER_MODE mode = TPM2_ALG_NULL;
     TPM2B_IV ivIn = DUMMY_2B_DATA16(.buffer);
     TPM2B_MAX_BUFFER *outData;
     TPM2B_IV *ivOut;
@@ -899,7 +774,7 @@ test_Hash(void **state)
 
     TPM2B_MAX_BUFFER data = DUMMY_2B_DATA(.buffer);
     TPMI_ALG_HASH hashAlg = TPM2_ALG_SHA1;
-    TPMI_RH_HIERARCHY hierarchy = TPM2_RH_OWNER;
+    ESYS_TR hierarchy = ESYS_TR_RH_OWNER;
     TPM2B_DIGEST *outHash;
     TPMT_TK_HASHCHECK *validation;
     r = Esys_Hash(esys_context,
@@ -1033,7 +908,7 @@ test_SequenceComplete(void **state)
 
     ESYS_TR sequenceHandle_handle = DUMMY_TR_HANDLE_KEY;
     TPM2B_MAX_BUFFER buffer = DUMMY_2B_DATA(.buffer);
-    TPMI_RH_HIERARCHY hierarchy = TPM2_RH_OWNER;
+    ESYS_TR hierarchy = ESYS_TR_RH_OWNER;
     TPM2B_DIGEST *result;
     TPMT_TK_HASHCHECK *validation;
     r = Esys_SequenceComplete(esys_context,
@@ -1921,7 +1796,7 @@ test_HierarchyControl(void **state)
     Esys_GetTcti(esys_context, &tcti);
 
     ESYS_TR authHandle_handle = DUMMY_TR_HANDLE_HIERARCHY_PLATFORM;
-    TPMI_RH_ENABLES enable = TPM2_RH_OWNER;
+    ESYS_TR enable = ESYS_TR_RH_OWNER;
     TPMI_YES_NO state2 = 0;
     r = Esys_HierarchyControl(esys_context,
                               authHandle_handle,
